unit fColumnDefinition;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtCtrls,
  oScriptColumn;

type
  TfrmColumnDefinition = class(TForm)
    cbxField: TComboBox;
    rgType: TRadioGroup;
    edtComment: TEdit;
    edtXMLTag: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    cbxInternal: TCheckBox;
    Label3: TLabel;
    btnCancel: TButton;
    btnOK: TButton;
    procedure FormCreate(Sender: TObject);
    procedure cbxOkToClose(Sender: TObject);
    procedure cbxFieldChange(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmColumnDefinition: TfrmColumnDefinition;

function NewColumn: TScriptColumn;
function EditColumn(aScriptColumn: TScriptColumn): boolean;

implementation

uses
  fMain,
  oFMField,
  oScriptTypes;

{$R *.dfm}

function NewColumn: TScriptColumn;
begin
  with TfrmColumnDefinition.Create(Application) do
    try
      Caption := 'Create New Column Definition';
      if ShowModal = mrOK then
        begin
          Result := TScriptColumn.Create;
          if cbxField.ItemIndex > -1 then
            Result.FMFieldNumber := TFMField(cbxField.Items.Objects[cbxField.ItemIndex]).Number
          else
            Result.FMFieldNumber := cbxField.Text; {Custom expression}
          Result.XMLTag := edtXMLTag.Text;
          Result.FMFieldType := TFMFieldType(rgType.ItemIndex);
          Result.FInternalFormat := cbxInternal.Checked;
          Result.Comment := edtComment.Text;
        end
      else
        Result := nil;
    finally
      Free;
    end;
end;

function EditColumn(aScriptColumn: TScriptColumn): boolean;
var
  i: integer;
begin
  with TfrmColumnDefinition.Create(Application) do
    try
      Caption := 'Update Column Definition';
      cbxField.Text := aScriptColumn.FMFieldNumber;
      for i := 0 to cbxField.Items.Count - 1 do
        if TFMField(cbxField.Items.Objects[i]).Number = aScriptColumn.FMFieldNumber then
          begin
            cbxField.ItemIndex := i;
            Break;
          end;
      edtXMLTag.Text := aScriptColumn.XMLTag;
      rgType.ItemIndex := Ord(aScriptColumn.FMFieldType);
      cbxInternal.Checked := aScriptColumn.FInternalFormat;
      edtComment.Text := aScriptColumn.Comment;

      if ShowModal = mrOK then
        begin
          if cbxField.ItemIndex > -1 then
            aScriptColumn.FMFieldNumber := TFMField(cbxField.Items.Objects[cbxField.ItemIndex]).Number
          else
            aScriptColumn.FMFieldNumber := cbxField.Text; {Custom expression}
          aScriptColumn.XMLTag := edtXMLTag.Text;
          aScriptColumn.FMFieldType := TFMFieldType(rgType.ItemIndex);
          aScriptColumn.FInternalFormat := cbxInternal.Checked;
          aScriptColumn.Comment := edtComment.Text;
          Result := True;
        end
      else
        Result := False;
    finally
      Free;
    end;
end;

procedure TfrmColumnDefinition.FormCreate(Sender: TObject);
var
  aType: TFMFieldType;
begin
  cbxField.Items.Assign(fCurrentFile.Fields);
  for aType := Low(TFMFieldType) to High(TFMFieldType) do
    rgType.Items.Add(FIELD_TYPE_NAME[aType]);
  edtXMLTag.Text := '';
  rgType.ItemIndex := Ord(ftFreeText);
  cbxInternal.Checked := False;
  edtComment.Text := '';
  btnOK.Enabled := False;
end;

procedure TfrmColumnDefinition.cbxOkToClose(Sender: TObject);
begin
  btnOK.Enabled :=
    (cbxField.Text <> '') and
    (edtXMLTag.Text <> '') and
    (rgType.ItemIndex > -1);
end;

procedure TfrmColumnDefinition.cbxFieldChange(Sender: TObject);
begin
  if cbxField.ItemIndex > -1 then
    with TFMField(cbxField.Items.Objects[cbxField.ItemIndex]) do
      begin
        edtXMLTag.Text := DefaultXMLTag;
        rgType.ItemIndex := Ord(DataType);
        edtComment.Text := Format('Fileman Field ''%s''', [DescriptiveName]);
      end;
  cbxOkToClose(Sender);
end;

end.

