unit oScript;

interface

uses
  Classes,
  XMLIntf,
  XMLDoc,
  oScriptColumn,
  oScriptTypes;

type
  TScript = class(TObject)
  private
    fXMLNode: IXMLNode;

    function getAttribute(aName: TScriptAttribute): string;
    procedure setAttribute(aName: TScriptAttribute; aNewValue: string);

    function getValue(aName: TScriptValue): string;
    procedure setValue(aName: TScriptValue; aNewValue: string);

    function getScriptType: TScriptType;
    procedure setScriptType(aScriptType: TScriptType);

  public
    function Rename(aNewName: string): boolean;
    function ColumnNode: IXMLNode;
    function GetVistAScript(aList: TStrings): integer;

    property XMLNode: IXMLNode read fXMLNode write fXMLNode;

    property Name: string index saName read getAttribute write setAttribute;

    property ScriptType: TScriptType read getScriptType write setScriptType;

    property FMRoot: string index svFMRoot read getValue write setValue;
    property FMFile: string index svFMFile read getValue write setValue;
    property MExecute: string index svMExecute read getValue write setValue;
    property MWhere: string index svMWhere read getValue write setValue;
    property FMIndex: string index svFMIndex read getValue write setValue;
    property Cached: string index svCached read getValue write setValue;

    property Comment: string index svComment read getValue write setValue;
  end;

function NewScript(aXMLNode: IXMLNode): TScript; overload;
function NewScript(aXMLDocument: IXMLDocument; aName: string; aScriptType: TScriptType = stQuery): TScript; overload;

implementation

uses
  SysUtils,
  Variants;

function NewScript(aXMLNode: IXMLNode): TScript;
var
  aType: TScriptType;
begin
  Result := TScript.Create;
  Result.fXMLNode := aXMLNode;
end;

function NewScript(aXMLDocument: IXMLDocument; aName: string; aScriptType: TScriptType = stQuery): TScript; overload;
begin
  Result := TScript.Create;
  Result.fXMLNode := aXMLDocument.DocumentElement.AddChild('SCRIPT');
  Result.Name := aName;
  Result.ScriptType := aScriptType;
  Result.Comment := 'New Script';
end;

{ TScript }

function TScript.getAttribute(aName: TScriptAttribute): string;
begin
  try
    Result := VarToStrDef(fXMLNode.Attributes[SCRIPT_ATTRIBUTE[aName]], '');
  except
    Result := '';
  end;
end;

procedure TScript.setAttribute(aName: TScriptAttribute; aNewValue: string);
begin
  fXMLNode.Attributes[SCRIPT_ATTRIBUTE[aName]] := aNewValue;
end;

function TScript.getValue(aName: TScriptValue): string;
begin
  try
    Result := VarToStrDef(fXMLNode.ChildValues[SCRIPT_VALUE[aName]], '');
  except
    Result := '';
  end;
end;

procedure TScript.setValue(aName: TScriptValue; aNewValue: string);
begin
  fXMLNode.ChildValues[SCRIPT_VALUE[aName]] := aNewValue;
end;

function TScript.getScriptType: TScriptType;
begin
  try
    Result := TScriptType(fXMLNode.Attributes['TYPE']);
  except
    for Result := Low(TScriptType) to High(TScriptType) do
      if fXMLNode.Attributes['TYPE'] = SCRIPT_TYPE[Result] then
        Exit;
    Result := stQuery;
  end;
end;

procedure TScript.setScriptType(aScriptType: TScriptType);
begin
  fXMLNode.Attributes['TYPE'] := aScriptType;
end;

function TScript.Rename(aNewName: string): boolean;
begin
  if aNewName <> '' then
    begin
      fXMLNode.Attributes['NAME'] := aNewName;
      Result := True;
    end
  else
    Result := False;
end;

function TScript.ColumnNode: IXMLNode;
begin
  Result := fXMLNode.ChildNodes.FindNode('COLUMNS');
  if not Assigned(Result) then { Build a new one :) }
    Result := fXMLNode.AddChild('COLUMNS');
end;

function TScript.GetVistAScript(aList: TStrings): integer;
var
  aNode: IXMLNode;
  aLine: string;
begin
  aList.Clear;
  aList.Add(
    Format(
    '%s;%s;%s;%s;%s;%s;%s',
    [
    SCRIPT_TYPE[getScriptType],
      getValue(svFMFile),
      getValue(svMExecute),
      getValue(svFMRoot),
      getValue(svMWhere),
      getValue(svFMIndex),
      getValue(svCached)
      ]
      ));
  aNode := ColumnNode.ChildNodes.First;
  while Assigned(aNode) do
    begin
      with NewScriptColumn(aNode) do
        try
          aLine := FMFieldNumber;
          aLine := Format('%s;%s', [aLine, XMLTag]);
          if FInternalFormat then
            aLine := Format('%s;%s', [aLine, 'I'])
          else
            aLine := Format('%s;%s', [aLine, 'E']);
          aLine := Format('%s;%s', [aLine, FIELD_TYPE[FMFieldType]]);
          aLine := Format('%s;%s', [aLine, Comment]);
          aList.Add(aLine);
        finally
          Free;
        end;
      aNode := aNode.NextSibling;
    end;
end;

end.

