unit oTermError;

interface

uses
  //oTerm,
  oTermComponent,
  uXML,
  ComCtrls,
  XMLIntf,
  XMLDoc,
  SysUtils,
  Classes,
  iCPTerminology;

type
  TTermError = class(TTermComponent, ITermError)
  private
    fErrorType: TTermErrorType;
    fErrorMessage: string;
    fDateTimeFound: TDateTime;
  public
    constructor Create(aParentTerm: ITerm);
    destructor Destroy; override;

    function getErrorType: TTermErrorType; stdcall;
    function getErrorMessage: string; stdcall;
    function getDateTimeFound: TDateTime; stdcall;
    function getComponentDisplayName: string; override; stdcall;
    function getDetailLine: string; override; stdcall;

    procedure setErrorType(const aValue: TTermErrorType); stdcall;
    procedure setErrorMessage(const aValue: string); stdcall;
    procedure setDateTimeFound(const aValue: TDateTime); stdcall;

    procedure Load(aXMLNode: IXMLNode); override; stdcall;
    procedure Save(aXMLNode: IXMLNode); override; stdcall;

    function AddToListView(aListView: TListView): TListItem; override; stdcall;
  end;

implementation

const
  {Default XML Tags for the properties}
  XML_ERROR_TYPE = 'ERROR_TYPE';
  XML_ERROR_MESSAGE = 'ERROR_MESSAGE';
  XML_DATE_TIME_FOUND = 'ERROR_DATE_TIME_FOUND';

  {Default values for the properties}
  DEFAULT_ERROR_TYPE = etUnknown;
  DEFAULT_ERROR_MESSAGE = 'No Description of Error Available';

constructor TTermError.Create(aParentTerm: ITerm);
begin
  inherited Create(ctError, aParentTerm, aParentTerm);
  fErrorType := DEFAULT_ERROR_TYPE;
  fErrorMessage := DEFAULT_ERROR_MESSAGE;
  fDateTimeFound := Now;
end;

destructor TTermError.Destroy;
begin
  inherited;
end;

function TTermError.AddToListView(aListView: TListView): TListItem;
{Override of default adder}
begin
  Result := aListView.Items.Add;
  Result.Caption := Format('%s (%s)', [getParentTerm.getTerm, TERM_TYPE_DISPLAY[getParentTerm.getTermType]]);
  Result.SubItems.Add(TERM_ERROR_TYPE[fErrorType]);
  Result.SubItems.Add(fErrorMessage);
  Result.Data := Self;
end;

function TTermError.getComponentDisplayName: string;
begin
  Result := Format('%-10s: %s', [TERM_ERROR_TYPE[fErrorType], fErrorMessage]);
end;

function TTermError.getDateTimeFound: TDateTime;
begin
  Result := fDateTimeFound;
end;

function TTermError.getDetailLine: string;
begin
  Result := Format('  %-20s %s', [TERM_ERROR_TYPE[fErrorType] + ':', fErrorMessage]);
end;

function TTermError.getErrorMessage: string;
begin
  Result := fErrorMessage;
end;

function TTermError.getErrorType: TTermErrorType;
begin
  Result := fErrorType;
end;

procedure TTermError.Load(aXMLNode: IXMLNode);
begin
  try
    inherited Load(aXMLNode);
    fErrorType := TTermErrorType(XMLValueAsInt(aXMLNode, XML_ERROR_TYPE, Ord(DEFAULT_ERROR_TYPE)));
    fErrorMessage := XMLValueAsStr(aXMLNode, XML_ERROR_MESSAGE, DEFAULT_ERROR_MESSAGE);
    fDateTimeFound := XMLValueAsDateTime(aXMLNode, XML_DATE_TIME_FOUND);
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to load %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

procedure TTermError.Save(aXMLNode: IXMLNode);
begin
  try
    inherited Save(aXMLNode);
    aXMLNode.ChildValues[XML_ERROR_TYPE] := fErrorType;
    aXMLNode.ChildValues[XML_ERROR_MESSAGE] := fErrorMessage;
    aXMLNode.ChildValues[XML_DATE_TIME_FOUND] := DateTimeToXMLString(fDateTimeFound);
  except
    on E: Exception do
      raise ETerminologyLoadError.CreateFmt('Unable to save %s. Error: %s', [Self.ClassName, E.Message]);
  end;
end;

procedure TTermError.setDateTimeFound(const aValue: TDateTime);
begin
  fDateTimeFound := aValue;
end;

procedure TTermError.setErrorMessage(const aValue: string);
begin
  fErrorMessage := aValue;
end;

procedure TTermError.setErrorType(const aValue: TTermErrorType);
begin
  fErrorType := aValue;
end;

end.

