unit uHomePage;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, uPatients, uDevices, uScenario, StdCtrls, ComCtrls, ExtCtrls,
  IdBaseComponent, IdHL7, OleCtrls, SHDocVw, uMessageLogItem, Mask, uServers;

type
  TfraHomePage = class(TFrame)
    hl7VistAConnection: TIdHL7;
    pnlCenter: TPanel;
    Label3: TLabel;
    pnlConnectionSettings: TPanel;
    btnStart: TButton;
    btnStop: TButton;
    cbSaveMessages: TCheckBox;
    sbMessagesSent: TScrollBox;
    Panel1: TPanel;
    Label2: TLabel;
    Bevel2: TBevel;
    tvWard: TTreeView;
    pnlTop: TPanel;
    Label1: TLabel;
    Bevel1: TBevel;
    Label4: TLabel;
    Label6: TLabel;
    lblConnectionStatus: TLabel;
    cbScenario: TComboBox;
    cbxServer: TComboBox;
    btnConnect: TButton;
    procedure cbScenarioSelect(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure hl7VistAConnectionConnect(Sender: TObject);
    procedure hl7VistAConnectionDisconnect(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
  private
    { Private declarations }
    FPatientList: TPatientList;
    FDeviceList: TDeviceList;
    FScenarioList: TScenarioList;
    FServerList: TServerList;

    FActiveScenario: TScenario;
    procedure Log(const AObservation: TTimedObservationItem; const AMessage: string; AResponse: TSendResponse);
    function GenerateMessage(AObsItem: TTimedObservationItem): string;
    procedure SendData(Sender: TTimedObservationItem);
  protected
    procedure SetScenarioList(const AScenarioList: TScenarioList);
    procedure SetPatientList(const APatientList: TPatientList);
    procedure SetDeviceList(const ADeviceList: TDeviceList);
    procedure SetServerList(const AServerList: TServerList);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure RefreshServerList;
    property PatientList: TPatientList read FPatientList write SetPatientList;
    property DeviceList: TDevicelist read FDeviceList write SetDeviceList;
    property ScenarioList: TScenarioList read FScenarioList write SetScenarioList;
    property ServerList: TServerList read FServerList write SetServerList;
  end;

implementation

{$R *.dfm}

uses
  SyncObjs, IdException, uConstants;

var
  CritSec: TCriticalSection;

{ TfraHomePage }

procedure TfraHomePage.SetDeviceList(const ADeviceList: TDeviceList);
begin
  FDeviceList := ADeviceList;
end;

procedure TfraHomePage.SetPatientList(const APatientList: TPatientList);
begin
  FPatientList := APatientList;
end;

procedure TfraHomePage.SetScenarioList(const AScenarioList: TScenarioList);
var
  I: Integer;
begin
  FScenarioList := AScenarioList;
  if Assigned(FScenarioList) then
  begin
    cbScenario.Clear;
    cbScenario.Items.BeginUpdate;
    try
      for I := 0 to FScenarioList.Count - 1 do
      begin
        cbScenario.AddItem(FScenarioList[I].Ward.Name, FScenarioList[I]);
      end;
    finally
      cbScenario.Items.EndUpdate;
    end;

  end;
end;

procedure TfraHomePage.SetServerList(const AServerList: TServerList);
var
  Enum: TCollectionEnumerator;
  Server: TServer;
begin
  cbxServer.Clear;

  if assigned(AServerList) then
  begin
    Enum := AServerList.GetEnumerator;
    while Enum.MoveNext do
    begin
      Server := TServer(Enum.GetCurrent);
      cbxServer.AddItem(Server.DisplayName, Server);
    end;
  end;
end;

procedure TfraHomePage.btnConnectClick(Sender: TObject);
var
  Server: TServer;
begin
  if hl7VistAConnection.Connected then
  begin
    try
      hl7VistAConnection.Stop;
    except on E:EIDClosedSocket do
      {
        There is a special circle of hell reserved for twits that insist
        on throwing exceptions for normal circumstances.
      }
    end;
    btnConnect.Caption := 'Connect';
  end
  else
  begin
    if cbxServer.ItemIndex > -1 then
    begin
      Server := TServer(cbxServer.Items.Objects[cbxServer.ItemIndex]);
      hl7VistAConnection.Address := Server.ServerIP;
      hl7VistAConnection.Port := Server.ServerPort;
      hl7VistAConnection.Start;
      btnConnect.Caption := 'Disconnect';
    end;
  end;
end;

procedure TfraHomePage.btnStartClick(Sender: TObject);
begin
  FActiveScenario.Start;
end;

procedure TfraHomePage.btnStopClick(Sender: TObject);
begin
  FActiveScenario.Stop;
end;

procedure TfraHomePage.cbScenarioSelect(Sender: TObject);
var
  I, J: Integer;
  Node: TTreeNode;
  PtNode: TTreeNode;
  DevNodes: TTreeNode;
  DevNode: TTreeNode;
begin
  FActiveScenario := TScenario(cbScenario.Items.Objects[cbScenario.ItemIndex]);
  tvWard.Items.BeginUpdate;
  try
    tvWard.Items.Clear;
    if Assigned(FActiveScenario) then
    begin
      for I := 0 to FActiveScenario.Ward.Count - 1 do
      begin
        Node := tvWard.Items.AddChild(nil, 'Bed ' + IntToStr(I));
        Node.Data := FActiveScenario.Ward[I];

        PtNode := tvWard.Items.AddChild(Node, 'Patient: ' + FActiveScenario.Ward[I].Patient.FullName);
        PtNode.Data := FActiveScenario.Ward[I].Patient;

        DevNodes := tvWard.Items.AddChild(Node, 'Devices');
        for J := 0 to FActiveScenario.Ward[I].Devices.Count - 1 do
        begin
          DevNode := tvWard.Items.AddChild(DevNodes, 'Device: ' + FActiveScenario.Ward[I].Devices[J].Description);
          DevNode.Data := FActiveScenario.Ward[I].Devices[J];

        end;
        FActiveScenario.Ward[I].OnObservation := SendData;
        Node.AlphaSort(true);
      end;
    end;
  finally
    tvWard.Items.EndUpdate;
  end;
end;

procedure TfraHomePage.SendData(Sender: TTimedObservationItem);
var
  Data: string;
begin
  CritSec.Enter;
  try
    {
      The concept here is that each observation type on each device will hit this
      method.  This will be the method that actually invokes the HL7 connection
      component and sends the HL7 data.  Note that this HAS to be in a critical
      section since, even though it isn't technically threaded, we could have
      the system tripping over itself if the timer for a new observation fired while
      we were still dealing with an older transaction.  This Would Be Bad(tm).
      So, we have to serialize everything as it comes in.
    }
    Data := GenerateMessage(Sender);

    try

      Log(Sender, Data, hl7VistAConnection.AsynchronousSend(Data));
      //Log(Sender, Data, srOK);
    except on E:Exception do
      begin
        MessageDlg(E.Message, mtError, [mbOK], 0);
      end;
    end;
  finally
    CritSec.Leave;
  end;
end;

procedure TfraHomePage.Log(const AObservation: TTimedObservationItem;  const AMessage: string; AResponse: TSendResponse);
var
  fraMessageLogItem: TfraMessageLogItem;
  Control: TControl;
  Patient: TPatient;
  Device: TDevice;
  Observation: TObservation;
  Logfile: TextFile;
begin
  {
    We need to append the message to the log file.
  }
  if cbSaveMessages.Checked then
  begin  
    AssignFile(Logfile, 'HL7Log.txt');
    if FileExists('HL7Log.txt') then
      Append(Logfile)
    else
      Rewrite(Logfile);
    WriteLn(Logfile, AMessage);
    Flush(Logfile);
    Close(Logfile);
  end;

  {
    Add a new frame to the panel
  }
  fraMessageLogItem := TfraMessageLogItem.Create(sbMessagesSent);
  fraMessageLogItem.Parent := sbMessagesSent;
  fraMessageLogItem.Name := 'MessageLogItem' + IntToStr(sbMessagesSent.ControlCount + 1);
  Control := sbMessagesSent.Controls[sbMessagesSent.ControlCount - 1];
  fraMessageLogItem.Top := Control.Top + Control.Height + 2;

  Patient := AObservation.Patient;
  Observation := AObservation.Observation;
  Device := AObservation.Device;

  {
    Create the Caption on the panel
  }
  Caption := Patient.FirstName + ' ' + Patient.LastName + ' -> ' +
    Device.Description + ' ';
  if assigned(Observation) then
    Caption := Caption + '(' + Observation.Description + '): ';

  case AResponse of
  srNone:
    Caption := Caption + 'No response.';
  srNoConnection:
    Caption := Caption + 'No connection.';
  srSent, srOK:
      // As far as we can tell, everything went okay.
    Caption := Caption + 'Sent okay.';
  srTimeout:
    Caption := Caption + 'Timeout.';
  else
    //srError and any other type of error.
    Caption := Caption + 'Unknown error.';
  end;

  fraMessageLogItem.lblMessageHeader.Caption := Caption;

  {
    Add the acutal message to the memo on the new frame.
  }
  fraMessageLogItem.mmoMessage.Lines.Add(StringReplace(AMessage, #13, #13#10, [rfReplaceAll]));
  fraMessageLogItem.ResizeMemo;

end;

procedure TfraHomePage.RefreshServerList;
begin
  SetServerList(FServerList);
end;

function TfraHomePage.GenerateMessage(AObsItem: TTimedObservationItem): string;
var
  Patient: TPatient;
  Device: TDevice;
  Observation: TObservation;
  Enum: TCollectionEnumerator;
begin
  Patient := AObsItem.Patient;
  Observation := AObsItem.Observation;
  Device := AObsItem.Device;
  result := Device.Header +  #13 +
            Patient.Segment;
  if Trim(Device.ORCSegment) <> '' then
    result := result + #13 + Replace(Device.ORCSegment);

  if Trim(Device.OBRSegment) <> '' then
    result := result + #13 + Replace(Device.OBRSegment);
    
  if Device.SendAllObservationsAtOnce then
  begin
    Enum := Device.Observations.GetEnumerator;
    while Enum.MoveNext do
    begin
      result := result + #13 + TObservation(Enum.GetCurrent).Segment;
    end;
  end
  else
  begin
    result := result + Observation.Segment;
  end;
  result := result + #13;
end;


procedure TfraHomePage.hl7VistAConnectionConnect(Sender: TObject);
begin
  lblConnectionStatus.Caption := 'Connected';
end;

procedure TfraHomePage.hl7VistAConnectionDisconnect(Sender: TObject);
begin
  lblConnectionStatus.Caption := 'Not connected';
end;

constructor TfraHomePage.Create(AOwner: TComponent);
begin
  inherited;
end;

destructor TfraHomePage.Destroy;
begin
  inherited;
end;

initialization
  CritSec := TCriticalSection.Create;

finalization
  FreeAndNil(CritSec)

end.
